#!/usr/bin/env python

"""
Read summary output produced by noninteractive-alignment-panel.py and group
matched subjects.

This is currently only useful when you are matching against a subject protein
database whose titles look e.g., like this:

gi|820945251|ref|YP_009137096.1| envelope glycoprotein H [Human herpesvirus 1]
gi|820945301|ref|YP_009137146.1| virion protein US10 [Human herpesvirus 1]
gi|820945229|ref|YP_009137074.1| ubiquitin E3 ligase ICP0 [Human herpesvirus 1]

In this case, those three matched subjects are proteins from the same virus.
This script will gather those matches under their common "Human herpesvirus 1"
title and print them together.

Reads from standard input, writes to standard output.
"""

from __future__ import print_function

import sys
import re
from collections import defaultdict
from operator import itemgetter

SUBJECT_RE = re.compile('^([^\[]+)\[([^\]]+)\]$')

subjectTitles = defaultdict(list)
proteinsWithNoSubject = []

for index, proteinLine in enumerate(sys.stdin):
    proteinLine = proteinLine[:-1]
    (coverage, medianScore, bestScore, readCount, hspCount, subjectLength,
     subjectTitle) = proteinLine.split('\t')

    match = SUBJECT_RE.match(subjectTitle)
    if match:
        proteinTitle = match.group(1).strip()
        subjectTitle = match.group(2)
        subjectTitles[subjectTitle].append({
            'coverage': float(coverage),
            'index': index,
            'medianScore': float(medianScore),
            'bestScore': float(bestScore),
            'readCount': int(readCount),
            'hspCount': int(hspCount),
            'proteinTitle': proteinTitle,
        })
    else:
        proteinsWithNoSubject.append(proteinLine)

titleGetter = itemgetter('proteinTitle')
readCountGetter = itemgetter('readCount')

for subjectTitle in sorted(subjectTitles):
    proteinMatches = subjectTitles[subjectTitle]
    proteinCount = len(proteinMatches)
    totalReads = sum(readCountGetter(p) for p in proteinMatches)
    print('%s (%d protein%s, %d read%s)' %
          (subjectTitle,
           proteinCount, '' if proteinCount == 1 else 's',
           totalReads, '' if totalReads == 1 else 's'))
    proteinMatches.sort(key=titleGetter)
    for proteinMatch in proteinMatches:
        print('  %(coverage)f\t%(medianScore)f\t%(bestScore)f\t'
              '%(readCount)d\t%(hspCount)d\t%(index)d\t%(proteinTitle)s' %
              proteinMatch)
    print()



#!/usr/bin/env python

"""
Read summary output produced by group-summary-proteins.py from many samples
and group it by virus.

Reads filenames from standard input, writes to standard output.

The filenames on stdin will typically be the 05-panel/summary-virus files.
E.g.,

    $ find . -name summary-virus | group-summary-viruses-html.py
"""

from __future__ import print_function

import sys
import re
from os.path import dirname, join
from collections import defaultdict

SUMMARY_RE = re.compile(' (\(\d+ proteins?, \d+ reads?\))$')

viruses = defaultdict(list)
fileCount = 0

for filename in sys.stdin:
    filename = filename[:-1]
    fileCount += 1
    with open(filename) as fp:
        for line in fp:
            line = line[:-1]
            match = SUMMARY_RE.search(line)
            if match:
                virus = line[:-len(match.group(0))]
                counts = match.group(1)

                simpleFilename = filename
                if simpleFilename.startswith('./'):
                    simpleFilename = simpleFilename[2:]
                if simpleFilename.endswith('/05-panel/summary-virus'):
                    simpleFilename = simpleFilename[:-23]

                viruses[virus].append(
                    '&nbsp;&nbsp;<tt>%s %s</tt> '
                    '(<a href="%s">panel</a>)' %
                    (simpleFilename, counts,
                     join(dirname(filename), 'out', 'index.html')))
            elif line:
                viruses[virus].append((line, filename))

print('<p>Proteins from %d viruses were found across %d samples.</p>' %
      (len(viruses), fileCount))
print()

for virus in sorted(viruses):
    samples = viruses[virus]
    sampleCount = len([s for s in samples if not isinstance(s, tuple)])
    print('<p><strong>%s (in %d sample%s)</strong><br>' %
          (virus, sampleCount, '' if len(samples) == 1 else 's'))
    for sample in samples:
        if isinstance(sample, tuple):
            line, filename = sample
            fields = line.split('\t')
            index = int(fields[5])
            print(
                '&nbsp;&nbsp;&nbsp;&nbsp;<tt>%s</tt> '
                '(<a href="%s">blue plot</a>, <a href="%s">fasta</a>)<br>' %
                ('\t'.join(fields[:5] + fields[6:]),
                 join(dirname(filename), 'out', '%d.png' % index),
                 join(dirname(filename), 'out', '%d.fasta' % index)))
        else:
            print('%s<br>' % sample)
    print('</p>')



from unittest import TestCase
from six import StringIO

from dark.summarize import summarizeReads


class TestSummarizeReads(TestCase):
    """
    Test the summarizeReads function.
    """

    def testReadNumberEmptyInput(self):
        result = summarizeReads(StringIO(), 'fasta')
        self.assertEqual(result['read_number'], 0)

    def testReadNumberOneSequenceCount(self):
        seq = '>hey\nagtcagtcagtc'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['read_number'], 1)

    def testReadNumberTwoSequencesCount(self):
        seq = '>hey\nagtcagtcagtc\n>you\nacctg'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['read_number'], 2)

    def testTotalLengthEmptyInput(self):
        result = summarizeReads(StringIO(), 'fasta')
        self.assertEqual(result['total_length'], 0)

    def testTotalLengthOneString(self):
        seq = '>hey\nagtcagtcagtc'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['total_length'], 12)

    def testTotalLengthTwoStrings(self):
        seq = '>hey\nagtcagtcagtc\n>you\nacctg'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['total_length'], 17)

    def testBaseCountsEmptyImput(self):
        result = summarizeReads(StringIO(), 'fasta')
        self.assertEqual(result['base_counts'], {})

    def testBaseCountsOneRead(self):
        seq = '>hey\nagtcagtcagtc'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['base_counts'],
                         {'a': 3, 'c': 3, 't': 3, 'g': 3})

    def testBaseCountsTwoReads(self):
        seq = '>hey\nagtcagtcagtc\n>you\nacctg'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['base_counts'],
                         {'a': 4, 'c': 5, 't': 4, 'g': 4})

    def testMaxLengthListEmptyInput(self):
        result = summarizeReads(StringIO(), 'fasta')
        self.assertEqual(result['max_length'], 0)

    def testMaxLengthListTwoStrings(self):
        seq = '>hey\nagtcagtcagtc\n>you\nacctg'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['max_length'], 12)

    def testMinLengthListEmptyInput(self):
        result = summarizeReads(StringIO(), 'fasta')
        self.assertEqual(result['min_length'], 0)

    def testMinLengthListTwoStrings(self):
        seq = '>hey\nagtcagtcagtc\n>you\nacctg'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['min_length'], 5)

    def testMedianEmptyInput(self):
        result = summarizeReads(StringIO(), 'fasta')
        self.assertEqual(result['median_length'], 0)

    def testMedianOneString(self):
        seq = '>hey\nagtcagtcagtc'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['median_length'], 12)

    def testMedianThreeStrings(self):
        seq = '>hey\nagtcagtcagtc\n>you\nacctg\n>how\natgggtc'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['median_length'], 7)

    def testMedianFourStrings(self):
        seq = '>hey\nagtcagtcagtc\n>you\nacctg\n>how\natgggtc\n>are\n\
        atggctattgaactgtatct'
        result = summarizeReads(StringIO(seq), 'fasta')
        self.assertEqual(result['median_length'], 9.5)



#!/usr/bin/env python

from __future__ import print_function

import sys

from dark.aa import find
from dark.aa import CODONS

from dark.aa import AA_LETTERS, ALL_PROPERTIES, PROPERTY_NAMES


def findOrDie(s):
    """
    Look up an amino acid.

    @param s: A C{str} amino acid specifier. This may be a full name,
        a 3-letter abbreviation or a 1-letter abbreviation. Case is ignored.
    @return: An C{AminoAcid} instance, if one can be found. Else exit.
    """
    aa = find(s)
    if aa:
        return aa
    else:
        print('Unknown amino acid or codon: %s' % s, file=sys.stderr)
        print('Valid arguments are: %s.' % list(CODONS.keys()),
              file=sys.stderr)
        sys.exit(1)


if len(sys.argv) == 1:
    # Show info on all amino acides.
    aas = AA_LETTERS
else:
    aas = sys.argv[1:]

for aa in map(findOrDie, aas):
    print(aa.name)
    print('  3-letter abbreviation: %s' % aa.abbrev3)
    print('  1-letter abbreviation: %s' % aa.abbrev1)
    print('  Codons: %s' % ', '.join(sorted(aa.codons)))

    properties = []
    print('  Properties:', end=' ')
    for prop in ALL_PROPERTIES:
        if aa.properties & prop:
            properties.append(PROPERTY_NAMES[prop])
    print(', '.join(properties))

    print('  Property details:')
    for propertyDetail, value in aa.propertyDetails.items():
        print('    %s: %s' % (propertyDetail, value))



#!/usr/bin/env python

from __future__ import print_function

import sys
from os.path import basename

from dark.fastq import FastqReads

# Print a usage message if any arguments were given on the command
# line. This is to remind people who want to provide filenames that they
# must use '<' to supply our stdin and '>' to store our output.

if len(sys.argv) > 1:
    print('Usage: %s < input.fastq [> output.fasta]' % basename(sys.argv[0]),
          file=sys.stderr)
    sys.exit(1)
else:
    write = sys.stdout.write
    for read in FastqReads(sys.stdin):
        write(read.toString('fasta'))
